use super::GpsError;

use nmea::Nmea;

/// Data structure to store all relevant data collected from the gps
#[derive(Default)]
pub struct GpsData {
    pub nmea: Nmea,
}

impl ToString for GpsData {
    fn to_string(&self) -> String {
        format!(
            "Latitude: {:.4?} \nLongitude: {:.4?} \nSpeed: {:.1?}mph \nAltitude: {:.1?}m",
            self.nmea.latitude.unwrap_or_default(),
            self.nmea.longitude.unwrap_or_default(),
            knots_to_mph(self.nmea.speed_over_ground.unwrap_or_default()),
            self.nmea.altitude.unwrap_or_default(),
        )
    }
}

impl GpsData {
    /// Function to go through and update GPS data from a nmea stream
    ///
    /// Takes in a reference to a buffer of raw serial data and then
    /// tries to read it in as NMEA Data
    pub fn update(&mut self, buf: &Vec<u8>) -> Result<(), GpsError> {
        let nmea_raw = String::from_utf8(buf.to_owned())
            .map_err(|_| GpsError::ReadError())
            .unwrap_or_default();

        // Split at $ and then prepend the '$' back to each string
        let nmea_vec: Vec<String> = nmea_raw.split('$').map(|str| format!("${}", str)).collect();

        for nmea_string in nmea_vec {
            if nmea_string.is_empty() {
                continue;
            }

            match self.nmea.parse(&nmea_string) {
                Ok(sentence) => {
                    println!("Recieved GPS Packet: {}", sentence)
                }

                Err(e) => {
                    eprintln!("Couldn't parse Packet: {}", e)
                }
            }
        }

        Ok(())
    }
}

/// Function to simply convert knots to mph
pub fn knots_to_mph(knots: f32) -> f32 {
    knots * 1.150779
}
